package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.page.PageMethod;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.space.entity.*;
import pps.core.space.mapper.*;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.service.data.space_optimize_ultra_period.GetSpaceOptimizeUltraPeriodViewInput;
import pps.core.space.service.data.space_optimize_ultra_period.GetSpaceOptimizeUltraPeriodViewOutput;
import pps.core.space.service.data.space_optimize_ultra_period.QuerySpaceOptimizeUltraPeriodViewInput;
import pps.core.space.service.data.space_optimize_ultra_period.QuerySpaceOptimizeUltraPeriodViewOutput;
import pps.core.space.service.data.space_optimize_ultra_wellhead.GetSpaceOptimizeUltraWellheadOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 长期间开优化模块
 *
 * @author ZWT
 * @date 2024/03/22
 */
@XService
@XText("长期间开优化模块")
public class SpaceOptimizeUltraPeriodService extends SpaceOptimizeBaseService {

    /**
     * 长期间开优化--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySpaceOptimizeUltraPeriodViewOutput}>
     */
    @XApiGet
    @XText("长期间开优化--分页列表")
    public XPageResult<QuerySpaceOptimizeUltraPeriodViewOutput> querySpaceOptimizeUltraPeriodView(XContext context, QuerySpaceOptimizeUltraPeriodViewInput input) {
        SpaceOptimizeUltraPeriodViewMapper mapper = context.getBean(SpaceOptimizeUltraPeriodViewMapper.class);
        SpaceOptimizeUltraPeriodView record = XCopyUtils.copyNewObject(input, SpaceOptimizeUltraPeriodView.class);
        PageMethod.startPage(input.getPage(), input.getLimit());
        List<SpaceOptimizeUltraPeriodView> list = mapper.selectList(record);
        PageInfo<SpaceOptimizeUltraPeriodView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceOptimizeUltraPeriodViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceOptimizeUltraPeriodViewOutput.class);
        if (CollUtil.isNotEmpty(outputs)) {
            Map<String, String> gridTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE);
            Map<String, String> ouMap = ServiceUtil.getOuMapByOuIds(context, outputs.stream()
                    .map(QuerySpaceOptimizeUltraPeriodViewOutput::getOuId)
                    .distinct()
                    .collect(Collectors.toList())
            );
            String lineName = ServiceUtil.getPowerLineNameById(context, input.getLineId());
            outputs.forEach(output -> {
                output.setLineName(lineName);
                output.setOuName(ouMap.get(output.getOuId()));
                output.setGridTypeName(gridTypeMap.get(output.getGridTypeKey()));
                output.setOptimizeStateValue(output.getOptimizeState().equals(0) ? BusinessConstant.OPTIMIZED_C : BusinessConstant.NO_OPTIMIZATION_C);
            });
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 长期间开优化--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceOptimizeUltraPeriodViewOutput}>
     */
    @XApiGet
    @XText("长期间开优化--详情")
    public XSingleResult<GetSpaceOptimizeUltraPeriodViewOutput> getSpaceOptimizeUltraPeriodView(XContext context, GetSpaceOptimizeUltraPeriodViewInput input) {
        SpaceOptimizeUltraPeriodMapper mapper = context.getBean(SpaceOptimizeUltraPeriodMapper.class);
        String ultraPeriodId = input.getId();
        SpaceOptimizeUltraPeriodEnt period = mapper.selectById(ultraPeriodId);
        if (Objects.isNull(period)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceOptimizeUltraPeriodViewOutput output = XCopyUtils.copyNewObject(period, GetSpaceOptimizeUltraPeriodViewOutput.class);
        //查间开制度详情
        SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
        SpaceInstitutionDetailEnt detailEnt = detailMapper.selectById(period.getInstitutionId());
        if (StringUtils.equals(String.valueOf(BusinessConstant.ONE), Optional.ofNullable(detailEnt.getGridTypeKey())
                .orElse(String.valueOf(BusinessConstant.ZERO)))) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(period.getOptimizeDeadline());
            output.setStrategyDetailList(ServiceUtil.getStrategyDetailList(context,
                    GetBasePriceStrategyDetailInput.builder()
                            .lineId(period.getLineId())
                            .strategyMonth(String.valueOf(calendar.get(Calendar.MONTH) + 1))
                            .build()));
        } else {
            output.setStrategyDetailList(new ArrayList<>(0));
        }
        output.setDateList(ServiceUtil.getOptimizeDateList(period.getCreateTime(), period.getOptimizeDeadline()));
        output.setGridTypeKey(detailEnt.getGridTypeKey());
        //查井口列表
        SpaceOptimizeUltraWellheadMapper wellheadMapper = context.getBean(SpaceOptimizeUltraWellheadMapper.class);
        List<SpaceOptimizeUltraWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeUltraWellheadEnt>()
                .eq(SpaceOptimizeUltraWellheadEnt::getUltraPeriodId, ultraPeriodId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .orderByAsc(SpaceOptimizeUltraWellheadEnt::getStartSeq)
        );
        if (CollUtil.isNotEmpty(wellheadList)) {
            List<GetSpaceOptimizeUltraWellheadOutput> wellheadOutputs = XCopyUtils.copyNewList(wellheadList, GetSpaceOptimizeUltraWellheadOutput.class);
            //查井口配置列表
            SpaceOptimizeUltraDurationMapper durationMapper = context.getBean(SpaceOptimizeUltraDurationMapper.class);
            List<SpaceOptimizeUltraDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeUltraDurationEnt>()
                    .eq(SpaceOptimizeUltraDurationEnt::getUltraPeriodId, ultraPeriodId)
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .orderByAsc(SpaceOptimizeUltraDurationEnt::getIsOptimize, SpaceOptimizeUltraDurationEnt::getOpenWellTime)
            );
            if (CollUtil.isNotEmpty(durationList)) {
                List<SpaceOptimizeDateDuration> durationOutputs = XCopyUtils.copyNewList(durationList, SpaceOptimizeDateDuration.class);
                Map<String, List<SpaceOptimizeDateDuration>> collect = new HashMap<>(16);
                for (SpaceOptimizeDateDuration durationOutput : durationOutputs) {
                    if (StringUtils.isEmpty(durationOutput.getGenerationTypeKey())) {
                        durationOutput.setGenerationTypeKey(BusinessConstant.PHOTOVOLTAIC);
                    }
                    List<SpaceOptimizeDateDuration> tmpList = collect.get(durationOutput.getWellheadId());
                    if (CollUtil.isEmpty(tmpList)) {
                        tmpList = new ArrayList<>(16);
                        tmpList.add(durationOutput);
                        collect.put(durationOutput.getWellheadId(), tmpList);
                    } else {
                        tmpList.add(durationOutput);
                    }
                }
                wellheadOutputs.forEach(w -> {
                    if (collect.containsKey(w.getWellheadId())) {
                        Map<Integer, List<SpaceOptimizeDateDuration>> listMap = collect.get(w.getWellheadId()).stream()
                                .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getIsOptimize));
                        w.setOptimizeList(ServiceUtil.getAllDateDuration(listMap.get(0)));
                        w.setInitializeList(ServiceUtil.getAllDateDuration(listMap.get(1)));
                    } else {
                        w.setOptimizeList(new ArrayList<>(0));
                        w.setInitializeList(new ArrayList<>(0));
                    }
                });
            }
            output.setWellheadList(wellheadOutputs);
        } else {
            output.setWellheadList(new ArrayList<>(0));
        }
        //查功率预测信息
        output.setPowerGenerationList(
                super.getAveragePowerGenerationHourList(context, DynamicQueryPlantPredictedPowerInput.builder()
                        .plantIds(super.getOptimizeLineRelation(context, CollUtil.newArrayList(detailEnt))
                                .getPlantIdsByLineIdMap()
                                .get(detailEnt.getLineId()))
                        .startTime(DateUtil.beginOfDay(period.getCreateTime()).toString())
                        .endTime(DateUtil.beginOfDay(period.getOptimizeDeadline()).toString())
                        .dateType(BusinessConstant.ZERO)
                        .build())
        );
        return XSingleResult.success(output);
    }
}